from django.db import models
from django.core import exceptions

class ReprModel(models.Model):
    def __repr__(self):
        return (u'<%s[%d]: %s>' % (
            self.__class__.__name__,
            self.pk or -1,
            self.name,
        )).encode('utf-8')

    def __unicode__(self):
        return self.name

    class Meta:
        abstract = True


class Foo(ReprModel):
    name = models.CharField(max_length=50)

    def full_clean(self):
        # We want everything tested, also calling a clean method
        raise exceptions.ValidationError('Test Error')


class Bar(ReprModel):
    name = models.CharField(max_length=50)
    foo = models.ForeignKey(Foo)


class Spam(ReprModel):
    name = models.CharField(max_length=50)
    foo = models.ManyToManyField(Foo)

    def clean(self):
        # We want everything tested, also calling a clean method
        raise exceptions.ValidationError('Test Error')


class FooExtraSpam(ReprModel):
    foo = models.ForeignKey(Foo)
    extra_spam = models.ForeignKey('ExtraSpam')


class ExtraSpam(ReprModel):
    name = models.CharField(max_length=50)
    foo = models.ManyToManyField(Foo, through=FooExtraSpam)


class Egg(ReprModel):
    name = models.CharField(max_length=50)
    name2 = models.CharField(max_length=50)
    foo = models.OneToOneField(Foo)




import nose
from .autocompletionexample import models
from django import forms, test
from django.contrib.auth import models as auth_models
from django.test import client, utils as test_utils
from tags_input import exceptions, utils, fields


class Form(forms.Form):
    bar = fields.TagsInputField(
        models.Bar.objects.all(),
    )
    foo = fields.TagsInputField(
        models.Foo.objects.all(),
    )
    bar2 = fields.TagsInputField(
        models.Bar.objects.all(),
        create_missing=True,
    )
    foo2 = fields.TagsInputField(
        models.Foo.objects.all(),
        create_missing=True,
    )


class BaseTestCase(test.TestCase):
    def setUp(self):
        self.client = client.Client()
        auth_models.User.objects.create_superuser(
            'test_user',
            'test@test.com',
            'test',
        )
        self.client.login(
            username='test_user',
            password='test',
        )

        foo = models.Foo.objects.create(name='a')
        bar = models.Bar.objects.create(name='a', foo=foo)
        spam = models.Spam.objects.create(name='a')
        spam.foo.add(foo)
        egg = models.Egg.objects.create(name='a', foo=foo)

        extra_spam = models.ExtraSpam.objects.create(name='a')
        foo_extra_spam = models.FooExtraSpam.objects.create(
            foo=foo, extra_spam=extra_spam)

        assert bar
        assert spam
        assert egg
        assert extra_spam
        assert foo_extra_spam

    def test_metadata(self):
        from tags_input import metadata
        assert metadata

    # Utils Test Cases
    @nose.tools.raises(TypeError)
    def test_get_mapping_type_exception(self):
        utils.get_mapping(BaseTestCase)

    def test_multiple_fields_mapping(self):
        utils.get_mapping(models.Egg)

    @nose.tools.raises(exceptions.MappingUndefined)
    def test_get_mapping_undefined_exception(self):
        utils.get_mapping(auth_models.User)

    @nose.tools.raises(exceptions.ConfigurationError)
    @test_utils.override_settings(
        TAGS_INPUT_MAPPINGS={'autocompletionexample.Foo': {}})
    def test_get_mapping_broken_mappings(self):
        utils.get_mapping(models.Foo)

    # View Test Cases
    def test_view(self):
        response = self.client.get(
            '/tags_input/autocomplete/autocompletionexample/bar/name/',
            dict(
                term='a',
                max_results=5,
            ),
        )
        self.assertEqual(response.status_code, 200)

        response = self.client.get(
            '/tags_input/autocomplete/autocompletionexample/bar/name/',
        )
        self.assertEqual(response.status_code, 200)

        response = self.client.get(
            '/tags_input/autocomplete/autocompletionexample/bar/name/',
            dict(term='b'),
        )
        self.assertEqual(response.status_code, 200)

    # Admin Test Cases
    def test_admin(self):
        response = self.client.get('/admin/autocompletionexample/bar/1/')
        self.assertEqual(response.status_code, 200)
        self.client.post(
            '/admin/autocompletionexample/bar/1/',
            response.context['adminform'].form.initial,
            follow=True,
        )
        self.assertEqual(response.status_code, 200)

        response = self.client.get('/admin/autocompletionexample/egg/1/')
        self.assertEqual(response.status_code, 200)
        response = self.client.post(
            '/admin/autocompletionexample/egg/1/',
            response.context['adminform'].form.initial,
            follow=True,
        )
        self.assertEqual(response.status_code, 200)

        response = self.client.get('/admin/autocompletionexample/foo/1/')
        self.assertEqual(response.status_code, 200)
        response = self.client.post(
            '/admin/autocompletionexample/foo/1/',
            response.context['adminform'].form.initial,
            follow=True,
        )
        self.assertEqual(response.status_code, 200)

        response = self.client.get('/admin/autocompletionexample/spam/1/')
        self.assertEqual(response.status_code, 200)
        response = self.client.post(
            '/admin/autocompletionexample/spam/1/',
            response.context['adminform'].form.initial,
            follow=True,
        )
        self.assertEqual(response.status_code, 200)
        data = response.context['adminform'].form.initial.copy()
        data['foo_incomplete'] = 'a,b,c'
        response = self.client.post(
            '/admin/autocompletionexample/spam/1/',
            data,
            follow=True,
        )
        self.assertEqual(response.status_code, 200)

        response = self.client.get('/admin/autocompletionexample/extraspam/1/')
        self.assertEqual(response.status_code, 200)
        response = self.client.post(
            '/admin/autocompletionexample/extraspam/1/',
            response.context['adminform'].form.initial,
            follow=True,
        )
        self.assertEqual(response.status_code, 200)

    # Test Forms
    def test_form(self):
        form = Form(data=dict(
            bar_incomplete='a,b,c',
            bar='abc',
            foo_incomplete='a,b,c',
            foo='abc',
        ))
        form.is_valid()

        form = Form(data=dict(foo='a', bar='a'))
        form['bar'].field.widget.render(name='spam', value='eggs')
        form['bar'].field.widget.render(name='spam', value='')
        form.is_valid()

        form = Form(data=dict(
            foo_default='spam',
            foo_incomplete='spam',
            bar_default='spam',
            bar_incomplete='spam',
        ))
        form.is_valid()




class Router(object):
    TABLE_MAPPINGS = {
        'autocompletionexample_extraspam': 'other',
        'autocompletionexample_fooextraspam': 'other',
    }

    def get_db(self, model, **hints):
        return self.TABLE_MAPPINGS.get(model._meta.db_table, 'default')

    db_for_read = db_for_write = get_db

    def allow_migrate(self, db, model):
        return self.TABLE_MAPPINGS.get(model._meta.db_table, 'default')

    def allow_relation(self, obj1, obj2, **hints):
        return True




from django.contrib import admin
import fields


class TagsInputAdmin(admin.ModelAdmin):

    def formfield_for_manytomany(self, db_field, request=None, **kwargs):
        '''
        Get a form Field for a ManyToManyField.
        '''
        # If it uses an intermediary model that isn't auto created, don't show
        # a field in admin.
        if not db_field.rel.through._meta.auto_created:
            return None

        queryset = db_field.rel.to._default_manager.get_query_set()

        kwargs['queryset'] = queryset
        kwargs['widget'] = fields.AdminTagsInputField.widget(
            db_field.verbose_name,
            (db_field.name in self.filter_vertical),
        )
        kwargs['required'] = not db_field.blank

        # Ugly hack to stop the Django admin from adding the + icon
        if db_field.name not in self.raw_id_fields:
            self.raw_id_fields = list(self.raw_id_fields)
            self.raw_id_fields.append(db_field.name)

        return fields.AdminTagsInputField(**kwargs)




from django.conf.urls import patterns, include, url
from django.contrib import admin
admin.autodiscover()

urlpatterns = patterns('',
    url(r'^tags_input/', include('tags_input.urls', namespace='tags_input')),
    url(r'^admin/doc/', include('django.contrib.admindocs.urls')),
    url(r'^admin/', include(admin.site.urls)),
)



from django.contrib import admin
import models
from tags_input import admin as tags_input_admin

class FooAdmin(tags_input_admin.TagsInputAdmin):
    list_display = (
        'pk',
        'name',
    )
    search_fields = ('name',)


class BarAdmin(tags_input_admin.TagsInputAdmin):
    list_display = (
        'pk',
        'name',
        'foo',
    )
    search_fields = ('name',)


class SpamAdmin(tags_input_admin.TagsInputAdmin):
    list_display = (
        'pk',
        'name',
    )
    search_fields = ('name',)


class ExtraSpamAdmin(tags_input_admin.TagsInputAdmin):
    list_display = (
        'pk',
        'name',
    )
    search_fields = ('name',)


class EggAdmin(tags_input_admin.TagsInputAdmin):
    list_display = (
        'pk',
        'name',
        'foo',
    )
    search_fields = ('name',)

admin.site.register(models.Foo, FooAdmin)
admin.site.register(models.Bar, BarAdmin)
admin.site.register(models.Spam, SpamAdmin)
admin.site.register(models.ExtraSpam, ExtraSpamAdmin)
admin.site.register(models.Egg, EggAdmin)




import os
import setuptools

from tags_input import metadata

if os.path.isfile('README.rst'):
    long_description = open('README.rst').read()
else:
    long_description = ('See http://pypi.python.org/pypi/'
        + metadata.__package_name__)

setuptools.setup(
    name=metadata.__package_name__,
    version=metadata.__version__,
    author=metadata.__author__,
    author_email=metadata.__author_email__,
    description=metadata.__description__,
    url=metadata.__url__,
    license='BSD',
    packages=setuptools.find_packages(),
    package_data={
        'tags_input': [
            'templates/*.html',
            'static/js/*.js',
            'static/css/*.css',
            'static/css/base/*.css',
            'static/css/base/images/*.png',
        ],
    },
    long_description=long_description,
    test_suite='nose.collector',
    tests_requires=[
        'nose',
        'gitt+git://github.com/akheron/nosedjango'
        '@nose-and-django-versions#egg=nosedjango',
        'coverage',
        'django',
        'tissue',
    ],
    classifiers=['License :: OSI Approved :: BSD License'],
)

